@@ -1,7 +1,14 @@
 from __future__ import print_function
 import pickle
 import re
-from werkzeug.routing import RequestRedirect, MethodNotAllowed, NotFound
+
+from werkzeug.routing import RequestRedirect
+
+try:
+    from werkzeug.exceptions import MethodNotAllowed, NotFound
+except ImportError:
+    from werkzeug.routing import MethodNotAllowed, NotFound
+
 import queue
 from nb2workflow import ontology, publish, schedule
 from nb2workflow.nbadapter import NotebookAdapter, find_notebooks, PapermillWorkflowIncomplete
@@ -24,14 +31,13 @@ from io import BytesIO
 
 
 from flask import Flask, make_response, jsonify, request, url_for, send_file, Response
-from flask.json import JSONEncoder
 from flask_caching import Cache
 from flask_cors import CORS
 
-from flasgger import LazyJSONEncoder, LazyString, Swagger, swag_from
-
+from flasgger import LazyString, Swagger, swag_from
 
 from nb2workflow.workflows import serialize_workflow_exception
+from nb2workflow.json import CustomJSONEncoder
 
 import threading
 
@@ -62,19 +68,6 @@ class ReverseProxied(object):
         return self.app(environ, start_response)
 
 
-class CustomJSONEncoder(LazyJSONEncoder):
-    def default(self, obj, *args, **kwargs):
-        try:
-            if isinstance(obj, type):
-                return dict(type_object=repr(obj))
-            iterable = iter(obj)
-        except TypeError:
-            pass
-        else:
-            return list(iterable)
-        return JSONEncoder.default(self, obj)
-
-
 cache = Cache(config={'CACHE_TYPE': 'simple'})
 
 
